# quiz.py
from string import ascii_lowercase
import random


NUM_QUESTIONS_PER_QUIZ = 5


def main(quecital_data):
    """
    Main function to run the quiz game.
    """
    questions = preprocess(quecital_data, num_questions=NUM_QUESTIONS_PER_QUIZ)

    num_correct = 0
    for index_label, question in enumerate(questions, start=1):
        print(f"\nQuestion {index_label}:")
        num_correct += main_process(question)

    print(f"\nYou got {num_correct} correct out of {len(questions)} questions")


def preprocess(data, num_questions):
    """
    Preprocess questions by loading from the TOML file and selecting a topic.

    Parameters:
    - path (pathlib.Path): The path to the TOML file containing questions.
    - num_questions (int): The number of questions to select for the quiz.

    Returns:
    List[dict]: A list of selected questions.
    """
    topics = {topic["label"]: topic["questions"] for topic in data.values()}
    topic_label = main_loop(
        question="Which topic do you want to be quizzed about",
        alternatives=sorted(topics),
    )[0]

    questions = topics[topic_label]
    num_questions = min(num_questions, len(questions))
    return random.sample(questions, k=num_questions)


def main_process(question):
    """
    Process a single question and get the user's response.

    Parameters:
    - question (dict): The question dictionary.

    Returns:
    int: 1 if the answer is correct, 0 otherwise.
    """
    correct_answers = question["answers"]
    alternatives = question["answers"] + question["alternatives"]
    shuffled_alternatives = random.sample(alternatives, k=len(alternatives))

    answers = main_loop(
        question=question["question"],
        alternatives=shuffled_alternatives,
        num_choices=len(correct_answers),
        hint=question.get("hint"),
    )

    if correct := (set(answers) == set(correct_answers)):
        print("⭐ Correct! ⭐")
    else:
        # adjust error message grammar based on number of expected correct
        # answers
        is_or_are = " is" if len(correct_answers) == 1 else "s are"
        print("\n- ".join([f"No, the answer{is_or_are}:"] + correct_answers))

    # display a message after the input answer is judged
    if "explanation" in question:
        print(f"\nEXPLANATION:\n{question['explanation']}")

    # performing an action after judging answer regardless of correctness
    # means you can't return inside the if ...  else block any longer.
    return 1 if correct else 0


def main_loop(question, alternatives, num_choices=1, hint=None):
    """
    Display the question and handle the user's input.

    Parameters:
    - question (str): The question to display.
    - alternatives (List[str]): List of answer alternatives.
    - num_choices (int): Number of choices the user needs to make.
    - hint (str): Optional hint to display.

    Returns:
    List[str]: A list of user's selected choices.
    """
    print(f"{question}?")
    labeled_alternatives = dict(zip(ascii_lowercase, alternatives))
    # check if hint not None, assign a label "?" to the value "Hint"
    if hint:
        labeled_alternatives["?"] = "Hint"

    for labeled_index, alternative in labeled_alternatives.items():
        print(f"  {labeled_index}) {alternative}")

    # rerun input prompt loop until return statement reached
    while True:
        plural_s = "" if num_choices == 1 else f"s (choose {num_choices})"
        answer = input(f"\nChoice{plural_s}? ")
        answers = set(answer.replace(",", " ").split())

        # Handle hint not None and "?" given as answer,
        #  i.e user wants to see hint
        if hint and "?" in answers:
            print(f"\nHINT: {hint}")
            continue

        # Handle incorrect quantity of answers input
        if len(answers) != num_choices:
            plural_s = "" if num_choices == 1 else "s, separated by comma"
            print(f"Please answer {num_choices} alternative{plural_s}")
            continue

        # Handle incorrect character(s) given as answer
        if any((invalid := answer) not in labeled_alternatives for answer in answers):
            print(
                f"{invalid!r} is not a valid choice. "
                f"Please use {', '.join(labeled_alternatives)}"
            )
            continue

        return [labeled_alternatives[answer] for answer in answers]



import click
from pathlib import Path
import tomllib
import random
from quecital.multiline_text_editor import get_multiline_edit


def main(data):
    assets = preprocess(data)
    attempt = get_attempt(assets)
    display = f"\
        \nThe prompt was:\
        \n{assets['question']}\n\
        \nThe attempt was:\
        \n{attempt}\n\
        \nThe correct answer is:\
        \n{assets['answers']}\n"
    if "explanation" in assets and assets["explanation"]:
        display += f"\n\nExplanation: {assets['explanation']}"
    click.echo(display)

def compose_prompt(assets, show_hint=False):
    hint = "Hint not provided"
    if "hint" in assets and assets["hint"] and show_hint is False:
        hint = "<DELETE FOR HINT>"
    if "hint" in assets and assets["hint"] and show_hint is True:
        hint = assets['hint']
    prompt = f'The prompt is:\n\n{assets["question"]}\n\n{hint}'
    return prompt


def get_attempt(assets):
    prompt = compose_prompt(assets)
    attempt = get_multiline_edit(prompt)
    if attempt == 'show_hint':
        prompt = compose_prompt(assets, show_hint=True)
        attempt = get_multiline_edit(prompt, hint_shown=True)

    return attempt


def preprocess(quecital_data):
    topics = list(quecital_data.keys())
    prompt = "\nFrom which topic will you perform a random recital\n\n"
    for index, topic in enumerate(topics, start=1):
        prompt += f"\n{index}: {topic}"
    prompt += '\n\nEnter one of the following ->'
    valid_choices = list(map(str, range(1, len(topics) + 1)))
    selected_index = click.prompt(prompt, type=click.Choice(valid_choices))
    topic = topics[int(selected_index) - 1]
    return random.choice(quecital_data[topic]["recitals"])


if __name__ == '__main__':
    quecital_path = Path("quecital.toml")
    data = tomllib.loads(quecital_path.read_text(encoding="utf-8"))
    main(data)


# add_recitals.py


from tomli_w import dump
from quecital.quiz import main_loop
from quecital.multiline_text_editor import get_multiline_edit


def main(quecital_toml_path, quecital_data):
    questions_list = path_to_list_of_dicts(quecital_data)
    new_question_dict = form_toml_entry()
    questions_list.append(new_question_dict)
    with open(quecital_toml_path, "wb") as f:
        dump(quecital_data, f)


def path_to_list_of_dicts(quecital_data):
    """
    Return the list of dictionaries representing questions for the
        selected topic.

    Returns:
        List[Dict]: A list of dictionaries where each dictionary represents
            a question.
    """
    topic_label = main_loop(
        question="Which topic do you want to add a recital to",
        alternatives=quecital_data.keys(),
    )[0]
    return quecital_data[topic_label]["recitals"]

#  TODO handle the key error if ["recitals"] not in quecital_data[topic_label]


def form_toml_entry():
    question = get_multiline_edit("Enter your prompt: ")
    answers = get_multiline_edit("Enter your recital: ")
    hint = get_multiline_edit("Offer a hint? -enter to skip")
    explanation = get_multiline_edit("Offer an explanation? - enter to skip")
    return {
        "question": question,
        "answers": answers,
        #"alternatives": list(alternatives),
        "hint": hint,
        "explanation": explanation,
    }



# add_quiz_question.py
import click
from tomli_w import dump
from quecital.quiz import main_loop
from quecital.multiline_text_editor import get_multiline_edit


def main(quecital_toml_path, quecital_data):
    questions_list = path_to_list_of_dicts(quecital_data)
    new_question_dict = form_toml_entry()
    questions_list.append(new_question_dict)
    with open(quecital_toml_path, "wb") as f:
        dump(quecital_data, f)


def path_to_list_of_dicts(quecital_data):
    """
    Return the list of dictionaries representing questions for the
        selected topic.

    Returns:
        List[Dict]: A list of dictionaries where each dictionary represents
            a question.
    """
    topic_label = main_loop(
        question="Which topic do you want to add a question to",
        alternatives=quecital_data.keys(),
    )[0]
    return quecital_data[topic_label]["questions"]


#  TODO handle the key error if ["questions"] not in quecital_data[topic_label]


def form_toml_entry():
    question = get_multiline_edit("Enter your question prompt: ")

    answers = answer()

    alternatives = alternative()

    hint = get_multiline_edit("Offer a hint? -enter to skip ")

    explanation = get_multiline_edit("Offer an explanation? - enter to skip ")

    return {
        "question": question,
        "answers": list(answers),
        "alternatives": list(alternatives),
        "hint": hint,
        "explanation": explanation,
    }


def answer():
    prompt = "Enter a correct answer: "
    answers = []  # to support questions with more than one required answer
    while True:
        answers.append(get_multiline_edit(prompt))
        add_another_answer = click.confirm(
            "Add a second required answer for this question?", default=False
        )
        if not add_another_answer:
            break
    return answers


def alternative():
    prompt = "Enter an alternative, i.e., an incorrect answer"
    alternatives = []
    while True:
        alternatives.append(get_multiline_edit(prompt))
        add_another_alternative = click.confirm(
            "Add another alternative?", default=True
        )
        if not add_another_alternative:
            break
    return alternatives



import click


def get_multiline_edit(prompt, hint_shown=False):
    """
    Capture multiline input from the user using an interactive editor.

    The function opens an interactive editor (specified by the user or the default)
    for multiline input. The user can edit the content and save the changes.

    Args:
        prompt (str): The prompt to display to the user.

    Returns:
        List[str]: A list of strings representing the multiline input.
    """
    # Custom marker to separate prompt text from user's input
    custom_marker = "# ---- Your Input Below ----\n"
    default_content = f"{prompt}\n\n{custom_marker}"
    hide_hint = '<DELETE FOR HINT>'
    no_hint = 'Hint not provided'

    edited_content = click.edit(
        text=default_content,
        editor=None,
    )  # Let Click choose the default editor
    # keep_open=False,  # Close the editor immediately after the user exits

    # an attempt after viewing the hint
    if edited_content and hint_shown is True:
        # Find the position of the custom marker and extract user's input
        marker_position = edited_content.find(custom_marker)
        user_input = edited_content[marker_position + len(custom_marker):].strip()
        return user_input

    # check if the user wants to see the hint
    if edited_content and hint_shown is False:
        if hide_hint not in edited_content and no_hint not in edited_content:
            return 'show_hint'
    # an attempt without viewing the hint
    if edited_content and hint_shown is False:
        if hide_hint in edited_content or no_hint in edited_content:
            # Find the position of the custom marker and extract user's input
            marker_position = edited_content.find(custom_marker)
            user_input = edited_content[marker_position + len(custom_marker):].strip()

            return user_input

    else:
        click.echo("No changes were made.")
        return ""


if __name__ == "__main__":
    # Example of using get_multiline_edit()
    code_input = get_multiline_edit("Enter your code:")

    if code_input:
        click.echo("\nYou entered:")
        for line in code_input:
            click.echo(line)



# quecital.py

from pathlib import Path
import tomllib
import click
from tomli_w import dump
from quecital import quiz, recital, add_quiz_question, add_recital


@click.group()
def quecital():
    """Quecital - Your Quiz App."""
    pass


def find_quecital_toml(quecital_storage):
    quecital_path = Path(quecital_storage)
    return quecital_path if quecital_path.is_file() else None


def new_topic():
    topic = input("Enter topic name: ").lower()
    label = topic.capitalize()
    return {topic: {"label": label, "questions": [], "recitals": []}}


def add_new_topic(topic, quecital_toml_path):
    quecital_toml = tomllib.loads(quecital_toml_path.read_text(encoding="utf-8"))
    quecital_toml.update(topic)
    with open(quecital_toml_path, "wb") as f:
        dump(quecital_toml, f)


def create_quecital_file(file_name):
    # Open the file in write mode, creating it if it doesn't exist
    with open(file_name, "wb") as f:
        dump(new_topic(), f)

    click.echo("A new quecital.toml was created.")


@quecital.command()
def start():
    # Check for quecital.toml in the cwd
    quecital_toml_path = find_quecital_toml("quecital.toml")

    if quecital_toml_path:
        click.echo(
            f"""
                    Welcome to Quecital!
                    Found quecital.toml at: {quecital_toml_path}
                    What would you like to do?
            """
        )
        quecital_data = tomllib.loads(quecital_toml_path.read_text(encoding="utf-8"))
        # Perform actions based on the existence of quecital.toml
        action = click.prompt(
            "1. Take a quiz\n2. Add a question\n3. Add a recital\n4. Start a recital\n5. Add a topic\n6. Exit",
            type=int,
        )

        match action:
            case 1:
                # Logic to start quiz
                click.echo("Starting quiz...")
                quiz.main(quecital_data)
            case 2:
                # Logic to add a question
                try:
                    while True:
                        click.echo("Adding a question...")
                        add_quiz_question.main(quecital_toml_path, quecital_data)
                        user_wants_to_add = click.confirm(
                            "Do you want to add another question?", default=True
                        )

                        if not user_wants_to_add:
                            break

                except KeyboardInterrupt:
                    click.echo("\nExiting the application.")
            case 3:
                # Logic to add a recital
                try:
                    while True:
                        click.echo("Adding a recital...")
                        add_recital.main(quecital_toml_path, quecital_data)
                        user_wants_to_add = click.confirm(
                            "Do you want to add another recital?", default=True
                        )

                        if not user_wants_to_add:
                            break

                except KeyboardInterrupt:
                    click.echo("\nExiting the application.")
            case 4:
                # Logic to start a recital
                click.echo("Starting recital")
                recital.main(quecital_data)
            case 5:
                # logic to add a topic to quecital.toml
                try:
                    while True:
                        click.echo("Adding a topic...")
                        add_new_topic(new_topic(), quecital_toml_path)
                        user_wants_to_add = click.confirm(
                            "Do you want to add another topic?", default=True
                        )

                        if not user_wants_to_add:
                            break

                except KeyboardInterrupt:
                    click.echo("\nExiting the application.")
            case 6:
                click.echo("Exiting Quecital. Goodbye!")
            case _:
                click.echo("Invalid choice. Exiting.")

    else:
        click.echo("quecital.toml not found in the current working directory.")
        # Perform actions when quecital.toml is not found
        create_file = click.confirm(
            "Do you want to create a new quecital file?", default=True
        )

        if create_file:
            create_quecital_file("quecital.toml")
